/*
 * Written by J.T. Conklin <jtc@netbsd.org>.
 * Change for long double by Jakub Jelinek <jj@ultra.linux.cz>
 * Public domain.
 */

/*
 * isinfl(x) returns 1 if x is inf, -1 if x is -inf, else 0;
 * no branching!
 */

#include "math_private.h"

#ifndef NO_LONG_DOUBLE
int
__isinfl (long double x)
{
	int32_t se, hx, lx;

	GET_LDOUBLE_WORDS(se, hx, lx, x);
	lx |= (hx & 0x7fffffffL) | ((se & 0x7fff) ^ 0x7fff);
	lx |= -lx;
	se &= 0x8000;
	return (int)(~(lx >> 31) & (1 - (se >> 14)));
}
weak_alias (__isinfl, isinfl)
#endif
